# A guessing game where the computer thinks of a
# 2-digit number and the user tries to guess it.
# Robust two-digit number-guessing game with hinting.

import random

# Prints an explanation of how to play the game.
def give_intro():
    print("Try to guess my two-digit number, and I'll tell you")
    print("how many digits from your guess appear in my number.")
    print()

# Returns number of matching digits between the two given numbers.
# pre: number and guess are unique two-digit numbers
def matches(number, guess):
    num_matches = 0
    if guess // 10 == number // 10 or guess // 10 == number % 10:
        num_matches += 1
    if guess // 10 != guess % 10 and (guess % 10 == number // 10 or
            guess % 10 == number % 10):
        num_matches += 1
    return num_matches

# Prompts until a number in proper range is entered.
# post: guess is between 0 and 99
def get_guess():
    guess = get_int("Your guess? ")
    while guess < 0 or guess > 99:
        print("Out of range; try again.")
        guess = get_int("Your guess? ")
    return guess

# Returns True if s can be converted to type int.
def is_integer(s):
    try:
        s = int(s)
        return True
    except ValueError:
        return False

# Repeatedly prompts for input until a numeric string is typed.
# Returns the int value that was typed by the user.
def get_int(prompt):
    line = input(prompt)
    while not is_integer(line):
        print("Not an integer; try again.")
        line = input(prompt) 
    return int(line)

def main():
    give_intro()

    # pick a random number from 0 to 99 inclusive
    number = random.randint(0, 99)

    # get first guess
    guess = get_guess()
    num_guesses = 1

    # give hints until correct guess is reached
    while guess != number:
        num_matches = matches(number, guess)
        print("Incorrect (hint:", num_matches, "digits match)")
        guess = get_guess()
        num_guesses += 1

    print("You got it right in", num_guesses, "tries.")
    
main()
