# This program uses a file of ZIP code information
# to allow a user to find ZIP codes within a
# certain distance of another ZIP code.

import math

# constant for radius of sphere (Earth), in miles
RADIUS = 3956.6

# Introduces the program to the user.
def intro():
    print("Welcome to the ZIP code database.")
    print("Give me a 5-digit ZIP code and a")
    print("proximity and I'll tell you where")
    print("that ZIP code is located along")
    print("with a list of other ZIP codes")
    print("within the given proximity.")
    print()

# Prompts the user and returns ZIP code and proximity.
def read_input():
    zip = input("ZIP code of interest? ")
    miles = float(input("Proximity in miles? "))
    print()
    return zip, miles


# Searches for the given string in the input file;
# if found, returns coordinates; else returns (0, 0).
def find_location(file, target_zip):
    for line in file:
        zip, lat, lng, city = line.rstrip().split(" : ")
        if zip == target_zip:
            print(zip, city)
            return float(lat), float(lng)
    
    # at this point we know the ZIP code isn't in the file
    return 0, 0

# Shows all matches for given coords within given distance.
def show_matches(file, lat1, lng1, miles):
    print("ZIP codes within", miles, "miles:")
    for line in file:
        zip, lat2, lng2, city = line.rstrip().split(" : ")
        dist = spherical_distance(lat1, lng1, \
                                  float(lat2), float(lng2))
        if dist <= miles:
            print(zip, city, round(dist, 2), "miles")

# Returns spherical distance in miles given the latitude 
# and longitude of two points (depends on constant RADIUS).
def spherical_distance(lat1, lng1, lat2, lng2):
    lat1 = math.radians(lat1)
    lng1 = math.radians(lng1)
    lat2 = math.radians(lat2)
    lng2 = math.radians(lng2)
    the_cos = math.sin(lat1) * math.sin(lat2) \
        + math.cos(lat1)  * math.cos(lat2) * math.cos(lng1 - lng2)
    arc_length = math.acos(the_cos)
    return arc_length * RADIUS

def main():
    intro()
    zip, miles = read_input()
    with open("zipcode.txt") as file:
        lat, lng = find_location(file, zip)
        file.seek(0)   # rewind file
        show_matches(file, lat, lng, miles)

main()
