# This program draws the Sierpinski Triangle fractal.
from DrawingPanel import *

# Returns a point that is halfway between the given two points.
def midpoint(p1, p2):
    x1, y1 = p1
    x2, y2 = p2
    x = (x1 + x2) // 2
    y = (y1 + y2) // 2
    return (x, y)

# Draws a Sierpinski Triangle figure of the given level with 
# its endpoints at the three given x/y points.
def sierpinski(panel, p1, p2, p3, level):
    if level == 1:
        # base case: draw a single filled triangle
        panel.fill_polygon(p1, p2, p3)
    else:
        # recursive case: split into 3 smaller triangles
        p4 = midpoint(p1, p2)
        p5 = midpoint(p2, p3)
        p6 = midpoint(p1, p3)
        sierpinski(panel, p1, p4, p6, level - 1)
        sierpinski(panel, p4, p2, p5, level - 1)
        sierpinski(panel, p6, p5, p3, level - 1)

def main():
    panel = DrawingPanel(600, 400)
    panel.fill_color = "black"
    p1 = (100, 30)
    p2 = (500, 30)
    p3 = (300, 376)
    level = 6
    sierpinski(panel, p1, p2, p3, level)

main()
