# Measures the runtime of an algorithm to find
# the range of values in a large list.
# First version.

import random
import time

# Swaps the values of the given list found at indexes i and j.
def swap(lst, i, j):
    if i != j:
        temp = lst[i]
        lst[i] = lst[j]
        lst[j] = temp


# Places the elements of the given list into sorted order
# using the merge sort algorithm.
# post: list is in sorted (nondecreasing) order
def merge_sort(lst):
    if len(lst) < 2:
        # base case: already trivially sorted
        return
    else:
        # recursive case: split list into two halves
        mid = len(lst) // 2
        left  = lst[ : mid]
        right = lst[mid : ]
        
        # (recursively) sort the two halves    
        merge_sort(left)
        merge_sort(right)
        
        # merge the two sorted halves back into the original list
        merge(lst, left, right)

# Merges sorted left/right lists into given result list.
# pre:  left and right must be sorted
# post: result contains merged sorted results of left and right
def merge(result, left, right):
    i1 = 0
    i2 = 0
    for i in range(len(result)):
        if i2 >= len(right) or i1 < len(left) and left[i1] <= right[i2]:
            result[i] = left[i1]    # take from left
            i1 += 1
        else:
            result[i] = right[i2]   # take from right
            i2 += 1

# Returns true if the given list is in sorted order.
def is_sorted(lst):
    for i in range(len(lst) - 1):
        if lst[i] > lst[i + 1]:
            return False
    return True        

# Creates and returns a random list of integers of length N.
def create_random_list(N):
    numbers = [0] * N
    for i in range(N):
        numbers[i] = random.randint(0, 999999999)
    return numbers

def main():
    # sort a random list of integers many times
    N = 500
    MAX_N = 8192000
    print("N", "runtime (ms)", sep="\t")
    while N <= MAX_N:
        numbers = create_random_list(N)
        start_time = time.time()
        r = merge_sort(numbers)
        elapsed_time = time.time() - start_time
        
        if not is_sorted(numbers):
            print("ERROR: sort failed")
        
        print(N, round(elapsed_time * 1000), sep="\t")
        N *= 2

main()
