# Measures the runtime of an algorithm to find
# the range of values in a large list.
# First version.

import random
import time

# Swaps the values of the given list found at indexes i and j.
def swap(lst, i, j):
    if i != j:
        temp = lst[i]
        lst[i] = lst[j]
        lst[j] = temp

# Places the elements of the given list into sorted order
# using the selection sort algorithm.
# post: list is in sorted (nondecreasing) order
def selection_sort(lst):
    for i in range(len(lst)):
        smallest = i
        for j in range(i + 1, len(lst)):
            if lst[j] < lst[smallest]:
                smallest = j
        swap(lst, i, smallest)

# Returns true if the given list is in sorted order.
def is_sorted(lst):
    for i in range(len(lst) - 1):
        if lst[i] > lst[i + 1]:
            return False
    return True        

# Creates and returns a random list of integers of length N.
def create_random_list(N):
    numbers = [0] * N
    for i in range(N):
        numbers[i] = random.randint(0, 999999999)
    return numbers

def main():
    # sort a random list of integers many times
    N = 500
    MAX_N = 32000
    print("N", "runtime (ms)", sep="\t")
    while N <= MAX_N:
        numbers = create_random_list(N)
        start_time = time.time()
        r = selection_sort(numbers)
        elapsed_time = time.time() - start_time
        
        if not is_sorted(numbers):
            print("ERROR: sort failed")
        
        print(N, round(elapsed_time * 1000), sep="\t")
        N *= 2

main()
