# Measures the runtime of an algorithm to
# sequentially search a list for a given value.
# First version.

import random
import time

# Sequential search algorithm.
# Returns the index at which the given target number first
# appears in the given input list, or -1 if it is not found.
def sequential_search(elements, target):
    for i in range(len(elements)):
        if elements[i] == target:
            return i
    return -1   # not found

# Creates and returns a random list of integers of length N.
def create_random_list(N):
    numbers = [0] * N
    for i in range(N):
        if i != N // 2:
            numbers[i] = random.randint(0, 999999999)
    return numbers

def create_sorted_list(N):
    numbers = [0] * N
    for i in range(N):
        if i != N // 2:
            numbers[i] = random.randint(0, 999999999)
    numbers.sort()
    return numbers

def main():
    # Search for a word's index many times
    N = 100000
    MAX_N = 32000000
    print("N", "runtime (ms)", sep="\t")
    while N <= MAX_N:
        numbers = create_random_list(N)
        
        start_time = time.time()
        index1 = sequential_search(numbers, 0)
        elapsed_time1 = time.time() - start_time
        
        start_time = time.time()
        index2 = sequential_search(numbers, -5)
        elapsed_time2 = time.time() - start_time
        
        print(N, round(elapsed_time1 * 1000), round(elapsed_time2 * 1000), sep="\t")
        N *= 2

main()
